@@ -20,11 +20,13 @@
 #include <google/protobuf/message.h>            // Message
 #include <google/protobuf/io/zero_copy_stream_impl_lite.h>
 #include <google/protobuf/io/coded_stream.h>
+#include <google/protobuf/text_format.h>
 #include "butil/logging.h"                       // LOG()
-#include "butil/time.h"
 #include "butil/iobuf.h"                         // butil::IOBuf
 #include "butil/raw_pack.h"                      // RawPacker RawUnpacker
 #include "butil/memory/scope_guard.h"
+#include "json2pb/json_to_pb.h"
+#include "json2pb/pb_to_json.h"
 #include "brpc/controller.h"                    // Controller
 #include "brpc/socket.h"                        // Socket
 #include "brpc/server.h"                        // Server
@@ -56,6 +58,8 @@ DEFINE_bool(baidu_protocol_use_fullname, true,
 DEFINE_bool(baidu_std_protocol_deliver_timeout_ms, false,
             "If this flag is true, baidu_std puts timeout_ms in requests.");
 
+DECLARE_bool(pb_enum_as_number);
+
 // Notes:
 // 1. 12-byte header [PRPC][body_size][meta_size]
 // 2. body_size and meta_size are in network byte order
@@ -137,23 +141,95 @@ ParseResult ParseRpcMessage(butil::IOBuf* source, Socket* socket,
     return MakeMessage(msg);
 }
 
+bool SerializeRpcMessage(const google::protobuf::Message& message,
+                         Controller& cntl, ContentType content_type,
+                         CompressType compress_type, butil::IOBuf* buf) {
+    auto serialize = [&](Serializer& serializer) -> bool {
+        bool ok;
+        if (COMPRESS_TYPE_NONE == compress_type) {
+            butil::IOBufAsZeroCopyOutputStream stream(buf);
+            ok = serializer.SerializeTo(&stream);
+        } else {
+            const CompressHandler* handler = FindCompressHandler(compress_type);
+            if (NULL == handler) {
+                return false;
+            }
+            ok = handler->Compress(serializer, buf);
+        }
+        return ok;
+    };
+
+    if (CONTENT_TYPE_PB == content_type) {
+        Serializer serializer([&message](google::protobuf::io::ZeroCopyOutputStream* output) -> bool {
+            return message.SerializeToZeroCopyStream(output);
+        });
+        return serialize(serializer);
+    } else if (CONTENT_TYPE_JSON == content_type) {
+        Serializer serializer([&message, &cntl](google::protobuf::io::ZeroCopyOutputStream* output) -> bool {
+            json2pb::Pb2JsonOptions options;
+            options.bytes_to_base64 = cntl.has_pb_bytes_to_base64();
+            options.jsonify_empty_array = cntl.has_pb_jsonify_empty_array();
+            options.always_print_primitive_fields = cntl.has_always_print_primitive_fields();
+            options.single_repeated_to_array = cntl.has_pb_single_repeated_to_array();
+            options.enum_option = FLAGS_pb_enum_as_number
+                                  ? json2pb::OUTPUT_ENUM_BY_NUMBER
+                                  : json2pb::OUTPUT_ENUM_BY_NAME;
+            std::string error;
+            bool ok = json2pb::ProtoMessageToJson(message, output, options, &error);
+            if (!ok) {
+                LOG(INFO) << "Fail to serialize message="
+                          << message.GetDescriptor()->full_name()
+                          << " to json :" << error;
+            }
+            return ok;
+        });
+        return serialize(serializer);
+    } else if (CONTENT_TYPE_PROTO_JSON == content_type) {
+        Serializer serializer([&message, &cntl](google::protobuf::io::ZeroCopyOutputStream* output) -> bool {
+            json2pb::Pb2ProtoJsonOptions options;
+            options.always_print_enums_as_ints = FLAGS_pb_enum_as_number;
+            AlwaysPrintPrimitiveFields(options) = cntl.has_always_print_primitive_fields();
+            std::string error;
+            bool ok = json2pb::ProtoMessageToProtoJson(message, output, options, &error);
+            if (!ok) {
+                LOG(INFO) << "Fail to serialize message="
+                          << message.GetDescriptor()->full_name()
+                          << " to proto-json :" << error;
+            }
+            return ok;
+        });
+        return serialize(serializer);
+    } else if (CONTENT_TYPE_PROTO_TEXT == content_type) {
+        Serializer serializer([&message](google::protobuf::io::ZeroCopyOutputStream* output) -> bool {
+            return google::protobuf::TextFormat::Print(message, output);
+        });
+        return serialize(serializer);
+    }
+    return false;
+}
+
 static bool SerializeResponse(const google::protobuf::Message& res,
-                              Controller& cntl, CompressType compress_type,
-                              butil::IOBuf& buf) {
+                              Controller& cntl, butil::IOBuf& buf) {
     if (res.GetDescriptor() == SerializedResponse::descriptor()) {
         buf.swap(((SerializedResponse&)res).serialized_data());
         return true;
     }
 
     if (!res.IsInitialized()) {
-        cntl.SetFailed(ERESPONSE,
-                       "Missing required fields in response: %s",
+        cntl.SetFailed(ERESPONSE, "Missing required fields in response: %s",
                        res.InitializationErrorString().c_str());
         return false;
-    } else if (!SerializeAsCompressedData(res, &buf, compress_type)) {
-        cntl.SetFailed(ERESPONSE,
-                       "Fail to serialize response, CompressType=%s",
-                       CompressTypeToCStr(compress_type));
+    }
+
+    ContentType content_type = cntl.response_content_type();
+    CompressType compress_type = cntl.response_compress_type();
+    if (!SerializeRpcMessage(res, cntl, content_type, compress_type, &buf)) {
+        cntl.SetFailed(
+            ERESPONSE, "Fail to serialize response=%s, "
+                       "ContentType=%s, CompressType=%s",
+            res.GetDescriptor()->full_name().c_str(),
+            ContentTypeToCStr(content_type),
+            CompressTypeToCStr(compress_type));
         return false;
     }
     return true;
@@ -184,12 +260,9 @@ struct BaiduProxyPBMessages : public RpcPBMessages {
 }
 
 // Used by UT, can't be static.
-void SendRpcResponse(int64_t correlation_id,
-                     Controller* cntl,
-                     RpcPBMessages* messages,
-                     const Server* server,
-                     MethodStatus* method_status,
-                     int64_t received_us) {
+void SendRpcResponse(int64_t correlation_id, Controller* cntl,
+                     RpcPBMessages* messages, const Server* server,
+                     MethodStatus* method_status, int64_t received_us) {
     ControllerPrivateAccessor accessor(cntl);
     Span* span = accessor.span();
     if (span) {
@@ -197,24 +270,29 @@ void SendRpcResponse(int64_t correlation_id,
     }
     Socket* sock = accessor.get_sending_socket();
 
-    std::unique_ptr<Controller, LogErrorTextAndDelete> recycle_cntl(cntl);
-    ConcurrencyRemover concurrency_remover(method_status, cntl, received_us);
+    const google::protobuf::Message* req = NULL == messages ? NULL : messages->Request();
+    const google::protobuf::Message* res = NULL == messages ? NULL : messages->Response();
+
+    // Recycle resources at the end of this function.
+    BRPC_SCOPE_EXIT {
+        {
+            // Remove concurrency and record latency at first.
+            ConcurrencyRemover concurrency_remover(method_status, cntl, received_us);
+        }
+
+        std::unique_ptr<Controller, LogErrorTextAndDelete> recycle_cntl(cntl);
 
-    auto messages_guard = butil::MakeScopeGuard([server, messages] {
         if (NULL == messages) {
             return;
         }
-        if (NULL != server->options().baidu_master_service) {
-            BaiduProxyPBMessages::Return(static_cast<BaiduProxyPBMessages*>(messages));
-        } else {
-            server->options().rpc_pb_message_factory->Return(messages);
-        }
-    });
 
-    const google::protobuf::Message* req = NULL == messages ? NULL : messages->Request();
-    const google::protobuf::Message* res = NULL == messages ? NULL : messages->Response();
-    ClosureGuard guard(brpc::NewCallback(
-        cntl, &Controller::CallAfterRpcResp, req, res));
+        cntl->CallAfterRpcResp(req, res);
+        if (NULL == server->options().baidu_master_service) {
+            server->options().rpc_pb_message_factory->Return(messages);
+        } else {
+            BaiduProxyPBMessages::Return(static_cast<BaiduProxyPBMessages*>(messages));
+        }
+    };
     
     StreamIds response_stream_ids = accessor.response_streams();
 
@@ -232,8 +310,7 @@ void SendRpcResponse(int64_t correlation_id,
     // response either
     CompressType compress_type = cntl->response_compress_type();
     if (res != NULL && !cntl->Failed()) {
-        append_body = SerializeResponse(
-            *res, *cntl, compress_type, res_body);
+        append_body = SerializeResponse(*res, *cntl, res_body);
     }
 
     // Don't use res->ByteSize() since it may be compressed
@@ -260,6 +337,7 @@ void SendRpcResponse(int64_t correlation_id,
     }
     meta.set_correlation_id(correlation_id);
     meta.set_compress_type(compress_type);
+    meta.set_content_type(cntl->response_content_type());
     if (attached_size > 0) {
         meta.set_attachment_size(attached_size);
     }
@@ -299,32 +377,37 @@ void SendRpcResponse(int64_t correlation_id,
         }
     }
 
+    ResponseWriteInfo args;
+    bthread_id_t response_id = INVALID_BTHREAD_ID;
     if (span) {
         span->set_response_size(res_buf.size());
+        CHECK_EQ(0, bthread_id_create(&response_id, &args, HandleResponseWritten));
     }
+
     // Send rpc response over stream even if server side failed to create
     // stream for some reason.
-    if(cntl->has_remote_stream()){
+    if (cntl->has_remote_stream()) {
         // Send the response over stream to notify that this stream connection
         // is successfully built.
         // Response_stream can be INVALID_STREAM_ID when error occurs.
         if (SendStreamData(sock, &res_buf,
                            accessor.remote_stream_settings()->stream_id(),
-                           response_stream_id) != 0) {
-            const int errcode = errno;
-            std::string error_text = butil::string_printf(64, "Fail to write into %s",
-                                                          sock->description().c_str());
-            PLOG_IF(WARNING, errcode != EPIPE) << error_text;
-            cntl->SetFailed(errcode,  "%s", error_text.c_str());
-            Stream::SetFailed(response_stream_ids, errcode, "%s",
-                              error_text.c_str());
+                           response_stream_id, response_id) != 0) {
+            error_code = errno;
+            PLOG_IF(WARNING, error_code != EPIPE)
+                << "Fail to write into " << sock->description();
+            cntl->SetFailed(error_code,  "Fail to write into %s",
+                            sock->description().c_str());
+            Stream::SetFailed(response_stream_ids, error_code,
+                              "Fail to write into %s",
+                              sock->description().c_str());
             return;
         }
 
         // Now it's ok the mark these server-side streams as connected as all the
         // written user data would follower the RPC response.
         // Reuse stream_ptr to avoid address first stream id again
-        if(stream_ptr) {
+        if (stream_ptr) {
             ((Stream*)stream_ptr->conn())->SetConnected();
         }
         for (size_t i = 1; i < response_stream_ids.size(); ++i) {
@@ -344,6 +427,10 @@ void SendRpcResponse(int64_t correlation_id,
         // users to set max_concurrency.
         Socket::WriteOptions wopt;
         wopt.ignore_eovercrowded = true;
+        if (INVALID_BTHREAD_ID != response_id) {
+            wopt.id_wait = response_id;
+            wopt.notify_on_success = true;
+        }
         if (sock->Write(&res_buf, &wopt) != 0) {
             const int errcode = errno;
             PLOG_IF(WARNING, errcode != EPIPE) << "Fail to write into " << *sock;
@@ -354,8 +441,10 @@ void SendRpcResponse(int64_t correlation_id,
     }
 
     if (span) {
+        bthread_id_join(response_id);
+        // Do not care about the result of background writing.
         // TODO: this is not sent
-        span->set_sent_us(butil::cpuwide_time_us());
+        span->set_sent_us(args.sent_us);
     }
 }
 
@@ -395,6 +484,71 @@ void EndRunningCallMethodInPool(
     return EndRunningUserCodeInPool(CallMethodInBackupThread, args);
 };
 
+bool DeserializeRpcMessage(const butil::IOBuf& data, Controller& cntl,
+                           ContentType content_type, CompressType compress_type,
+                           google::protobuf::Message* message) {
+    auto deserialize = [&](Deserializer& deserializer) -> bool {
+        bool ok;
+        if (COMPRESS_TYPE_NONE == compress_type) {
+            butil::IOBufAsZeroCopyInputStream stream(data);
+            ok = deserializer.DeserializeFrom(&stream);
+        } else {
+            const CompressHandler* handler = FindCompressHandler(compress_type);
+            if (NULL == handler) {
+                return false;
+            }
+            ok = handler->Decompress(data, &deserializer);
+        }
+        return ok;
+    };
+
+    if (CONTENT_TYPE_PB == content_type) {
+        Deserializer deserializer([message](
+            google::protobuf::io::ZeroCopyInputStream* input) -> bool {
+            return message->ParseFromZeroCopyStream(input);
+        });
+        return deserialize(deserializer);
+    } else if (CONTENT_TYPE_JSON == content_type) {
+        Deserializer deserializer([message, &cntl](
+            google::protobuf::io::ZeroCopyInputStream* input) -> bool {
+            json2pb::Json2PbOptions options;
+            options.base64_to_bytes = cntl.has_pb_bytes_to_base64();
+            options.array_to_single_repeated = cntl.has_pb_single_repeated_to_array();
+            std::string error;
+            bool ok = json2pb::JsonToProtoMessage(input, message, options, &error);
+            if (!ok) {
+                LOG(INFO) << "Fail to parse json to "
+                          << message->GetDescriptor()->full_name()
+                          << ": "<< error;
+            }
+            return ok;
+        });
+        return deserialize(deserializer);
+    } else if (CONTENT_TYPE_PROTO_JSON == content_type) {
+        Deserializer deserializer([message](
+            google::protobuf::io::ZeroCopyInputStream* input) -> bool {
+            json2pb::ProtoJson2PbOptions options;
+            options.ignore_unknown_fields = true;
+            std::string error;
+            bool ok = json2pb::ProtoJsonToProtoMessage(input, message, options, &error);
+            if (!ok) {
+                LOG(INFO) << "Fail to parse proto-json to "
+                          << message->GetDescriptor()->full_name()
+                          << ": "<< error;
+            }
+            return ok;
+        });
+        return deserialize(deserializer);
+    } else if (CONTENT_TYPE_PROTO_TEXT == content_type) {
+        Deserializer deserializer([message](
+            google::protobuf::io::ZeroCopyInputStream* input) -> bool {
+            return google::protobuf::TextFormat::Parse(input, message);
+        });
+        return deserialize(deserializer);
+    }
+    return false;
+}
+
 void ProcessRpcRequest(InputMessageBase* msg_base) {
     const int64_t start_parse_us = butil::cpuwide_time_us();
     DestroyingPtr<MostCommonMessage> msg(static_cast<MostCommonMessage*>(msg_base));
@@ -445,6 +599,7 @@ void ProcessRpcRequest(InputMessageBase* msg_base) {
     if (request_meta.has_timeout_ms()) {
         cntl->set_timeout_ms(request_meta.timeout_ms());
     }
+    cntl->set_request_content_type(meta.content_type());
     cntl->set_request_compress_type((CompressType)meta.compress_type());
     accessor.set_server(server)
         .set_security_mode(security_mode)
@@ -627,13 +782,17 @@ void ProcessRpcRequest(InputMessageBase* msg_base) {
                 cntl->request_attachment().swap(msg->payload);
             }
 
-            auto req_cmp_type = static_cast<CompressType>(meta.compress_type());
+            ContentType content_type = meta.content_type();
+            auto compress_type = static_cast<CompressType>(meta.compress_type());
             messages = server->options().rpc_pb_message_factory->Get(*svc, *method);
-            if (!ParseFromCompressedData(req_buf, messages->Request(), req_cmp_type)) {
-                cntl->SetFailed(EREQUEST, "Fail to parse request message, "
-                                          "CompressType=%s, request_size=%d",
-                                CompressTypeToCStr(req_cmp_type), req_size);
-                server->options().rpc_pb_message_factory->Return(messages);
+            if (!DeserializeRpcMessage(req_buf, *cntl, content_type,
+                                       compress_type, messages->Request())) {
+                cntl->SetFailed(
+                    EREQUEST, "Fail to parse request=%s, ContentType=%s, "
+                              "CompressType=%s, request_size=%d",
+                    messages->Request()->GetDescriptor()->full_name().c_str(),
+                    ContentTypeToCStr(content_type),
+                    CompressTypeToCStr(compress_type), req_size);
                 break;
             }
             req_buf.clear();
@@ -642,11 +801,9 @@ void ProcessRpcRequest(InputMessageBase* msg_base) {
         // `socket' will be held until response has been sent
         google::protobuf::Closure* done = ::brpc::NewCallback<
             int64_t, Controller*, RpcPBMessages*,
-            const Server*, MethodStatus*, int64_t>(&SendRpcResponse,
-                                                   meta.correlation_id(),
-                                                   cntl.get(), messages,
-                                                   server, method_status,
-                                                   msg->received_us());
+            const Server*, MethodStatus*, int64_t>(
+                &SendRpcResponse, meta.correlation_id(),cntl.get(),
+                messages, server, method_status, msg->received_us());
 
         // optional, just release resource ASAP
         msg.reset();
@@ -787,8 +944,7 @@ void ProcessRpcResponse(InputMessageBase* msg_base) {
         if (meta.has_attachment_size()) {
             if (meta.attachment_size() > res_size) {
                 cntl->SetFailed(
-                    ERESPONSE,
-                    "attachment_size=%d is larger than response_size=%d",
+                    ERESPONSE, "attachment_size=%d is larger than response_size=%d",
                     meta.attachment_size(), res_size);
                 break;
             }
@@ -798,20 +954,24 @@ void ProcessRpcResponse(InputMessageBase* msg_base) {
             cntl->response_attachment().swap(msg->payload);
         }
 
-        auto res_cmp_type = (CompressType)meta.compress_type();
-        cntl->set_response_compress_type(res_cmp_type);
+        ContentType content_type = meta.content_type();
+        auto compress_type = (CompressType)meta.compress_type();
+        cntl->set_response_content_type(content_type);
+        cntl->set_response_compress_type(compress_type);
         if (cntl->response()) {
             if (cntl->response()->GetDescriptor() == SerializedResponse::descriptor()) {
                 ((SerializedResponse*)cntl->response())->
                     serialized_data().append(*res_buf_ptr);
-            } else if (!ParseFromCompressedData(
-                    *res_buf_ptr, cntl->response(), res_cmp_type)) {
+            } else if (!DeserializeRpcMessage(*res_buf_ptr, *cntl, content_type,
+                                              compress_type, cntl->response())) {
                 cntl->SetFailed(
-                    ERESPONSE, "Fail to parse response message, "
-                    "CompressType=%s, response_size=%d", 
-                    CompressTypeToCStr(res_cmp_type), res_size);
+                    EREQUEST, "Fail to parse response=%s, ContentType=%s, "
+                              "CompressType=%s, request_size=%d",
+                    cntl->response()->GetDescriptor()->full_name().c_str(),
+                    ContentTypeToCStr(content_type),
+                    CompressTypeToCStr(compress_type), res_size);
             }
-        } // else silently ignore the response.        
+        } // else silently ignore the response.
     } while (0);
     // Unlocks correlation_id inside. Revert controller's
     // error code if it version check of `cid' fails
@@ -819,6 +979,33 @@ void ProcessRpcResponse(InputMessageBase* msg_base) {
     accessor.OnResponse(cid, saved_error);
 }
 
+void SerializeRpcRequest(butil::IOBuf* request_buf, Controller* cntl,
+                         const google::protobuf::Message* request) {
+    // Check sanity of request.
+    if (NULL == request) {
+        return cntl->SetFailed(EREQUEST, "`request' is NULL");
+    }
+    if (request->GetDescriptor() == SerializedRequest::descriptor()) {
+        request_buf->append(((SerializedRequest*)request)->serialized_data());
+        return;
+    }
+    if (!request->IsInitialized()) {
+        return cntl->SetFailed(EREQUEST, "Missing required fields in request: %s",
+                               request->InitializationErrorString().c_str());
+    }
+
+    ContentType content_type = cntl->request_content_type();
+    CompressType compress_type = cntl->request_compress_type();
+    if (!SerializeRpcMessage(*request, *cntl, content_type, compress_type, request_buf)) {
+        return cntl->SetFailed(
+            EREQUEST, "Fail to compress request=%s, "
+                      "ContentType=%s, CompressType=%s",
+            request->GetDescriptor()->full_name().c_str(),
+            ContentTypeToCStr(content_type),
+            CompressTypeToCStr(compress_type));
+    }
+}
+
 void PackRpcRequest(butil::IOBuf* req_buf,
                     SocketMessage**,
                     uint64_t correlation_id,
@@ -892,6 +1079,7 @@ void PackRpcRequest(butil::IOBuf* req_buf,
             request_meta->set_timeout_ms(accessor.real_timeout_ms());
         }
     }
+    meta.set_content_type(cntl->request_content_type());
 
     Span* span = accessor.span();
     if (span) {
@@ -907,5 +1095,20 @@ void PackRpcRequest(butil::IOBuf* req_buf,
     }
 }
 
+const char* ContentTypeToCStr(ContentType content_type) {
+    switch (content_type) {
+    case CONTENT_TYPE_PB:
+        return "pb";
+    case CONTENT_TYPE_JSON:
+        return "json";
+    case CONTENT_TYPE_PROTO_JSON:
+        return "proto-json";
+    case CONTENT_TYPE_PROTO_TEXT:
+        return "proto-text";
+    default:
+        return "unknown";
+    }
+}
+
 }  // namespace policy
 } // namespace brpc
