@@ -228,7 +228,7 @@ int TaskControl::init(int concurrency) {
         const int rc = pthread_create(&_workers[i], NULL, worker_thread, arg);
         if (rc) {
             delete arg;
-            LOG(ERROR) << "Fail to create _workers[" << i << "], " << berror(rc);
+            PLOG(ERROR) << "Fail to create _workers[" << i << "]";
             return -1;
         }
     }
@@ -272,8 +272,7 @@ int TaskControl::add_workers(int num, bthread_tag_t tag) {
                 &_workers[i + old_concurency], NULL, worker_thread, arg);
         if (rc) {
             delete arg;
-            LOG(WARNING) << "Fail to create _workers[" << i + old_concurency
-                         << "], " << berror(rc);
+            PLOG(WARNING) << "Fail to create _workers[" << i + old_concurency << "]";
             _concurrency.fetch_sub(1, butil::memory_order_release);
             break;
         }
