@@ -23,6 +23,7 @@
 #define BTHREAD_TYPES_H
 
 #include <stdint.h>                            // uint64_t
+#include "butil/macros.h"
 #if defined(__cplusplus)
 #include "butil/logging.h"                      // CHECK
 #endif
@@ -87,6 +88,7 @@ typedef struct {
     pthread_rwlock_t rwlock;
     void* list;
     void* free_keytables;
+    size_t size;
     int destroyed;
 } bthread_keytable_pool_t;
 
@@ -164,15 +166,38 @@ typedef struct {
     size_t sampling_range;
 } bthread_contention_site_t;
 
-typedef struct {
+struct mutex_owner_t {
+    bool hold;
+    uint64_t id;
+};
+
+typedef struct bthread_mutex_t {
+#if defined(__cplusplus)
+    bthread_mutex_t()
+        : butex(NULL), csite{}
+        , enable_csite(false)
+        , owner{false, 0} {}
+
+    DISALLOW_COPY_AND_ASSIGN(bthread_mutex_t);
+#endif
     unsigned* butex;
     bthread_contention_site_t csite;
+    bool enable_csite;
+    // Note: Owner detection of the mutex comes with average execution
+    // slowdown of about 50%, so it is only used for debugging and is
+    // only available when the macro `BRPC_DEBUG_LOCK' = 1.
+    mutex_owner_t owner;
 } bthread_mutex_t;
 
 typedef struct {
+    bool enable_csite;
 } bthread_mutexattr_t;
 
-typedef struct {
+typedef struct bthread_cond_t {
+#if defined(__cplusplus)
+    bthread_cond_t() : m(NULL), seq(NULL) {}
+    DISALLOW_COPY_AND_ASSIGN(bthread_cond_t);
+#endif
     bthread_mutex_t* m;
     int* seq;
 } bthread_cond_t;
@@ -180,7 +205,28 @@ typedef struct {
 typedef struct {
 } bthread_condattr_t;
 
-typedef struct {
+typedef struct bthread_sem_t {
+#if defined(__cplusplus)
+    bthread_sem_t() : butex(NULL), enable_csite(true) {}
+    DISALLOW_COPY_AND_ASSIGN(bthread_sem_t);
+#endif
+    unsigned* butex;
+    bool enable_csite;
+} bthread_sem_t;
+
+typedef struct bthread_rwlock_t {
+#if defined(__cplusplus)
+    bthread_rwlock_t()
+        : reader_count(0), reader_wait(0), wlock_flag(false), writer_csite{} {}
+    DISALLOW_COPY_AND_ASSIGN(bthread_rwlock_t);
+#endif
+    bthread_sem_t reader_sema; // Semaphore for readers to wait for completing writers.
+    bthread_sem_t writer_sema; // Semaphore for writers to wait for completing readers.
+    int reader_count; // Number of pending readers.
+    int reader_wait; // Number of departing readers.
+    bool wlock_flag; // Flag used to indicate that a write lock has been held.
+    bthread_mutex_t write_queue_mutex; // Held if there are pending writers.
+    bthread_contention_site_t writer_csite;
 } bthread_rwlock_t;
 
 typedef struct {
@@ -208,8 +254,10 @@ friend int bthread::bthread_once_impl(bthread_once_t* once_control, void (*init_
         INITIALIZED,
     };
 
+
     bthread_once_t();
     ~bthread_once_t();
+    DISALLOW_COPY_AND_ASSIGN(bthread_once_t);
 
 private:
     butil::atomic<int>* _butex;
