@@ -132,7 +132,10 @@ protected:
         // Hack: Regard `_server' as running 
         _server._status = brpc::Server::RUNNING;
         _server._options.auth = &_auth;
-        
+        if (!_server._options.rpc_pb_message_factory) {
+            _server._options.rpc_pb_message_factory = new brpc::DefaultRpcPBMessageFactory();
+        }
+
         EXPECT_EQ(0, pipe(_pipe_fds));
 
         brpc::SocketId id;
@@ -186,6 +189,21 @@ protected:
         return msg;
     }
 
+    brpc::policy::HttpContext* MakePostProtoJsonRequestMessage(const std::string& path) {
+        brpc::policy::HttpContext* msg = new brpc::policy::HttpContext(false);
+        msg->header().uri().set_path(path);
+        msg->header().set_content_type("application/proto-json");
+        msg->header().set_method(brpc::HTTP_METHOD_POST);
+
+        test::EchoRequest req;
+        req.set_message(EXP_REQUEST);
+        butil::IOBufAsZeroCopyOutputStream req_stream(&msg->body());
+        json2pb::Pb2ProtoJsonOptions options;
+        std::string error;
+        EXPECT_TRUE(json2pb::ProtoMessageToProtoJson(req, &req_stream, options, &error)) << error;
+        return msg;
+    }
+
     brpc::policy::HttpContext* MakePostProtoTextRequestMessage(
         const std::string& path) {
         brpc::policy::HttpContext* msg = new brpc::policy::HttpContext(false);
@@ -331,7 +349,7 @@ TEST_F(HttpTest, parse_http_address) {
 TEST_F(HttpTest, verify_request) {
     {
         brpc::policy::HttpContext* msg =
-                MakePostRequestMessage("/EchoService/Echo");
+            MakePostRequestMessage("/EchoService/Echo");
         VerifyMessage(msg, false);
         msg->Destroy();
     }
@@ -347,6 +365,12 @@ TEST_F(HttpTest, verify_request) {
         VerifyMessage(msg, false);
         msg->Destroy();
     }
+    {
+        brpc::policy::HttpContext* msg =
+            MakePostProtoJsonRequestMessage("/EchoService/Echo");
+        VerifyMessage(msg, false);
+        msg->Destroy();
+    }
     {
         brpc::policy::HttpContext* msg =
                 MakePostProtoTextRequestMessage("/EchoService/Echo");
@@ -730,15 +754,19 @@ private:
     butil::Status _destroying_st;
 };
 
+#ifdef BUTIL_USE_ASAN
+static const int GENERAL_DELAY_US = 1000000; // 1s
+#else
 static const int GENERAL_DELAY_US = 300000; // 0.3s
+#endif
 
 TEST_F(HttpTest, read_long_body_progressively) {
+    DownloadServiceImpl svc(DONE_BEFORE_CREATE_PA,
+                            std::numeric_limits<size_t>::max());
     butil::intrusive_ptr<ReadBody> reader;
     {
         const int port = 8923;
         brpc::Server server;
-        DownloadServiceImpl svc(DONE_BEFORE_CREATE_PA,
-                                std::numeric_limits<size_t>::max());
         EXPECT_EQ(0, server.AddService(&svc, brpc::SERVER_DOESNT_OWN_SERVICE));
         EXPECT_EQ(0, server.Start(port, NULL));
         {
@@ -820,12 +848,12 @@ TEST_F(HttpTest, read_short_body_progressively) {
 }
 
 TEST_F(HttpTest, read_progressively_after_cntl_destroys) {
+    DownloadServiceImpl svc(DONE_BEFORE_CREATE_PA,
+                            std::numeric_limits<size_t>::max());
     butil::intrusive_ptr<ReadBody> reader;
     {
         const int port = 8923;
         brpc::Server server;
-        DownloadServiceImpl svc(DONE_BEFORE_CREATE_PA,
-                                std::numeric_limits<size_t>::max());
         EXPECT_EQ(0, server.AddService(&svc, brpc::SERVER_DOESNT_OWN_SERVICE));
         EXPECT_EQ(0, server.Start(port, NULL));
         {
@@ -867,11 +895,11 @@ TEST_F(HttpTest, read_progressively_after_cntl_destroys) {
 
 TEST_F(HttpTest, read_progressively_after_long_delay) {
     butil::intrusive_ptr<ReadBody> reader;
+    DownloadServiceImpl svc(DONE_BEFORE_CREATE_PA,
+                            std::numeric_limits<size_t>::max());
     {
         const int port = 8923;
         brpc::Server server;
-        DownloadServiceImpl svc(DONE_BEFORE_CREATE_PA,
-                                std::numeric_limits<size_t>::max());
         EXPECT_EQ(0, server.AddService(&svc, brpc::SERVER_DOESNT_OWN_SERVICE));
         EXPECT_EQ(0, server.Start(port, NULL));
         {
@@ -916,10 +944,10 @@ TEST_F(HttpTest, read_progressively_after_long_delay) {
 }
 
 TEST_F(HttpTest, skip_progressive_reading) {
-    const int port = 8923;
-    brpc::Server server;
     DownloadServiceImpl svc(DONE_BEFORE_CREATE_PA,
                             std::numeric_limits<size_t>::max());
+    const int port = 8923;
+    brpc::Server server;
     EXPECT_EQ(0, server.AddService(&svc, brpc::SERVER_DOESNT_OWN_SERVICE));
     EXPECT_EQ(0, server.Start(port, NULL));
     brpc::Channel channel;
@@ -1026,6 +1054,7 @@ TEST_F(HttpTest, broken_socket_stops_progressive_reading) {
     ASSERT_EQ(ECONNRESET, reader->destroying_status().error_code());
 }
 
+#ifndef BUTIL_USE_ASAN
 static const std::string TEST_PROGRESSIVE_HEADER = "Progressive";
 static const std::string TEST_PROGRESSIVE_HEADER_VAL = "Progressive-val";
 
@@ -1141,6 +1170,8 @@ TEST_F(HttpTest, server_end_read_short_body_progressively) {
     ASSERT_FALSE(cntl.Failed());
 }
 
+// Fixme!!! Server progressive reader has a heap-use-after-free bug detected by ASan.
+// For details, see https://github.com/apache/brpc/issues/2145#issuecomment-2329413363
 TEST_F(HttpTest, server_end_read_failed) {
     const int port = 8923;
     brpc::ServiceOptions opt;
@@ -1177,6 +1208,7 @@ TEST_F(HttpTest, server_end_read_failed) {
     channel.CallMethod(NULL, &cntl, NULL, NULL, NULL);
     ASSERT_TRUE(cntl.Failed());
 }
+#endif // BUTIL_USE_ASAN
 
 TEST_F(HttpTest, http2_sanity) {
     const int port = 8923;
@@ -1666,7 +1698,6 @@ TEST_F(HttpTest, spring_protobuf_content_type) {
 
     brpc::Controller cntl2;
     test::EchoService_Stub stub(&channel);
-    req.set_message(EXP_REQUEST);
     res.Clear();
     cntl2.http_request().set_content_type("application/x-protobuf");
     stub.Echo(&cntl2, &req, &res, nullptr);
@@ -1759,7 +1790,7 @@ TEST_F(HttpTest, dump_http_request) {
     brpc::g_rpc_dump_sl.sampling_range = 0;
 }
 
-TEST_F(HttpTest, spring_protobuf_text_content_type) {
+TEST_F(HttpTest, proto_text_content_type) {
     const int port = 8923;
     brpc::Server server;
     EXPECT_EQ(0, server.AddService(&_svc, brpc::SERVER_DOESNT_OWN_SERVICE));
@@ -1784,6 +1815,55 @@ TEST_F(HttpTest, spring_protobuf_text_content_type) {
     ASSERT_TRUE(google::protobuf::TextFormat::ParseFromString(
             cntl.response_attachment().to_string(), &res));
     ASSERT_EQ(EXP_RESPONSE, res.message());
+
+    test::EchoService_Stub stub(&channel);
+    cntl.Reset();
+    cntl.http_request().set_content_type("application/proto-text");
+    res.Clear();
+    stub.Echo(&cntl, &req, &res, NULL);
+    ASSERT_FALSE(cntl.Failed());
+    ASSERT_EQ(EXP_RESPONSE, res.message());
+    ASSERT_EQ("application/proto-text", cntl.http_response().content_type());
+}
+
+TEST_F(HttpTest, proto_json_content_type) {
+    const int port = 8923;
+    brpc::Server server;
+    EXPECT_EQ(0, server.AddService(&_svc, brpc::SERVER_DOESNT_OWN_SERVICE));
+    EXPECT_EQ(0, server.Start(port, nullptr));
+
+    brpc::Channel channel;
+    brpc::ChannelOptions options;
+    options.protocol = "http";
+    ASSERT_EQ(0, channel.Init(butil::EndPoint(butil::my_ip(), port), &options));
+
+    brpc::Controller cntl;
+    test::EchoRequest req;
+    test::EchoResponse res;
+    req.set_message(EXP_REQUEST);
+    cntl.http_request().set_method(brpc::HTTP_METHOD_POST);
+    cntl.http_request().uri() = "/EchoService/Echo";
+    cntl.http_request().set_content_type("application/proto-json");
+    json2pb::Pb2ProtoJsonOptions json_options;
+    butil::IOBufAsZeroCopyOutputStream output_stream(&cntl.request_attachment());
+    ASSERT_TRUE(json2pb::ProtoMessageToProtoJson(req, &output_stream, json_options));
+    channel.CallMethod(nullptr, &cntl, nullptr, nullptr, nullptr);
+    ASSERT_FALSE(cntl.Failed()) << cntl.ErrorText();
+    ASSERT_EQ("application/proto-json", cntl.http_response().content_type());
+    json2pb::ProtoJson2PbOptions parse_options;
+    parse_options.ignore_unknown_fields = true;
+    butil::IOBufAsZeroCopyInputStream input_stream(cntl.response_attachment());
+    ASSERT_TRUE(json2pb::ProtoJsonToProtoMessage(&input_stream, &res, parse_options));
+    ASSERT_EQ(EXP_RESPONSE, res.message());
+
+    test::EchoService_Stub stub(&channel);
+    cntl.Reset();
+    cntl.http_request().set_content_type("application/proto-json");
+    res.Clear();
+    stub.Echo(&cntl, &req, &res, nullptr);
+    ASSERT_FALSE(cntl.Failed()) << cntl.ErrorText();
+    ASSERT_EQ(EXP_RESPONSE, res.message());
+    ASSERT_EQ("application/proto-json", cntl.http_response().content_type());
 }
 
 class HttpServiceImpl : public ::test::HttpService {
